/**
 *File name ubitx_Kenwood_CAT.cpp
 * CAT Support
 * 
 * This module handles serial communications with external program.
 * 
 */
#include <Arduino.h>
#include "ubitx.h"
#include "EEPROM.h"
#include <stdlib.h>

extern void printLine1(char *c);
extern void printLine2(char *c);
extern void printLine3(char *c);
extern void printLine4(char *c);
extern void updateDisplay(void);
extern void doAlignment(void);
extern int  btnDown(void);
extern void MenuExit(int btn);
extern void setFrequency(unsigned long xf);
extern void stopTx(void);
extern void startTx(byte txMode);
extern void menuSplitToggle( void );
extern void menuVfoToggle(void );
extern void ritDisable(void);
extern void ritEnable(void);
extern void startTx(byte txMode);
extern void stopTx(void);


extern unsigned long frequency;
extern volatile int ubitx_mode;
extern bool isUSB;
extern unsigned long vfoA, vfoB;
extern char vfoActive;
extern volatile bool keyDown;
extern unsigned long sideTone;
extern char splitOn;
extern char ritOn;
extern int wpm;

/////////////////////////////   CAT EEPROM ////////////////////////////////////
#define ACK 0

bool CAT_Tune;

static byte CAT_BUFF[132];
static int Cat_Buffer_Index;

///////////////////////////////////////////////////////////////////////////////

//Read EEProm by uBITX Manager Software
void ReadEEPRom(void){
  
  //5BYTES
  //CAT_BUFF[0] [1] [2] [3] [4] //4 COMMAND
  //0, 1 START ADDRESS
  uint16_t eepromStartIndex = CAT_BUFF[0] + CAT_BUFF[1] * 256;
  uint16_t eepromReadLength = CAT_BUFF[2] + CAT_BUFF[3] * 256;;
  byte checkSum = 0;
  byte read1Byte = 0;

  Serial.write(0x02); //STX
  checkSum = 0x02;
  for (uint16_t i = 0; i < eepromReadLength; i++)
  {
    read1Byte = EEPROM.read(eepromStartIndex + i);
    checkSum += read1Byte;
    Serial.write(read1Byte);
  }
  Serial.write(checkSum);
  Serial.write(ACK);
}

//Write just proecess 1byte
void WriteEEPRom(void){
  
  //5BYTES
  uint16_t eepromStartIndex = CAT_BUFF[0] + CAT_BUFF[1] * 256;
  byte write1Byte = CAT_BUFF[2];

  //Check Checksum
  if (CAT_BUFF[3] != ((CAT_BUFF[0] + CAT_BUFF[1] + CAT_BUFF[2]) % 256))
  {
    Serial.write(0x56); //CHECK SUM ERROR
    Serial.write(ACK);
  }
  else
  {
    EEPROM.write(eepromStartIndex, write1Byte);
    Serial.write(0x77); //OK  
    Serial.write(ACK);
  }
}


void Init_Cat(long baud, int portConfig)
{
  Serial.begin(baud, portConfig);
  Serial.flush();
  Cat_Buffer_Index = 0;
  CAT_Tune = false;
}




//////////////////////////////////////////////////////////////////////
void Process_Cat_Command( void ){
char Response_Buffer[132];
char Freq_Text[12];
int i;
char * temp;


 if( strncmp((char*)CAT_BUFF,"FA;",3) == 0){
   if( vfoActive == VFO_A )ultoa(frequency, Freq_Text, DEC);
    else  ultoa(vfoA, Freq_Text, DEC);
    strcpy(Response_Buffer,"FA");
    for(i=0;i<11-strlen(Freq_Text);i++)strcat(Response_Buffer,"0");
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FA",2) == 0){
    strncpy(Freq_Text,(char*)&CAT_BUFF[2],11);
    if( vfoActive == VFO_A ){
      frequency = strtol(Freq_Text,NULL,10);
      setFrequency(frequency);
    }else vfoA= strtol(Freq_Text,NULL,10);
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"FB;",3) == 0){
    if( vfoActive == VFO_B )ultoa(frequency, Freq_Text, DEC);
    else  ultoa(vfoB, Freq_Text, DEC);
    strcpy(Response_Buffer,"FB");
    for(i=0;i<11-strlen(Freq_Text);i++)strcat(Response_Buffer,"0");
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FB",2) == 0){
    strncpy(Freq_Text,(char*)&CAT_BUFF[2],11);
    if( vfoActive == VFO_B ){
      frequency = strtol(Freq_Text,NULL,10);
      setFrequency(frequency);
    }else vfoB= strtol(Freq_Text,NULL,10);
    updateDisplay();
       
  }else if( strncmp((char*)CAT_BUFF,"FR;",3) == 0){
    strcpy(Response_Buffer,"FR");
    if( vfoActive == VFO_A )strcat(Response_Buffer,"0");
    else  strcat(Response_Buffer,"1");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FR",2) == 0){
    if( CAT_BUFF[2] == '0' ) vfoActive == VFO_B;
    else vfoActive == VFO_A;
    menuVfoToggle();
    
  }else if( strncmp((char*)CAT_BUFF,"FT;",3) == 0){
    strcpy(Response_Buffer,"FT");
    if( splitOn )strcat(Response_Buffer,"1");
    else  strcat(Response_Buffer,"0");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"FT",2) == 0){
    if( CAT_BUFF[2] == '0' ) splitOn = true;
    else splitOn = false;
    menuSplitToggle();
    
  }else if( strncmp((char*)CAT_BUFF,"KS;",3) == 0){
    strcpy(Response_Buffer,"KS");
    itoa(wpm,Freq_Text,10);
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"KS",2) == 0){
    wpm = strtol((char*)&CAT_BUFF[2], &temp, 10);
    updateDisplay();
    
    
  }else if( strncmp((char*)CAT_BUFF,"uST;",4) == 0){
    strcpy(Response_Buffer,"uST");
    itoa(sideTone,Freq_Text,10);
    strcat(Response_Buffer,Freq_Text);
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);

  }else if( strncmp((char*)CAT_BUFF,"uST",3) == 0){
    sideTone = strtol((char*)&CAT_BUFF[3], &temp, 10);
    updateDisplay();
    
  }else if( strncmp((char*)CAT_BUFF,"MD;",3) == 0){
    strcpy(Response_Buffer,"MD");
    if( ubitx_mode == MODE_CW )strcat(Response_Buffer,"3");
    else{
      if( isUSB )strcat(Response_Buffer,"2");
      else strcat(Response_Buffer,"1");
    }
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);  
    
  }else if( strncmp((char*)CAT_BUFF,"MD",2) == 0){
    if( CAT_BUFF[2] == '1' ){
      isUSB = false;
      ubitx_mode = MODE_SSB;
      i = MODE_SSB;
    }else if( CAT_BUFF[2] == '2' ){
      isUSB = true;
      ubitx_mode = MODE_SSB;
      i = MODE_SSB;
    }else if( CAT_BUFF[2] == '3' ){
      ubitx_mode = MODE_CW;
      i = MODE_CW;
      isUSB = true;
    }else if( CAT_BUFF[2] == '7' ){
      ubitx_mode = MODE_CW;
      i = MODE_CW;
      isUSB = false;
    }
    EEPROM.put(MODE, i );
    setFrequency(frequency);
    updateDisplay();

  }else if( strncmp((char*)CAT_BUFF,"RT;",3) == 0){
    strcpy(Response_Buffer,"RT");
    if( ritOn )strcat(Response_Buffer,"1");
    else  strcat(Response_Buffer,"0");
    strcat(Response_Buffer,";");
    Serial.write(Response_Buffer);
   
  }else if( strncmp((char*)CAT_BUFF,"RT",2) == 0){
    if( CAT_BUFF[2] == '0' ) ritDisable();
    else ritEnable();
    updateDisplay();
    
   
  }else if( strncmp((char*)CAT_BUFF,"RX;",3) == 0){
    stopTx();
        
  }else if( strncmp((char*)CAT_BUFF,"TX0;",4) == 0){
    if( frequency >= 10000000 )isUSB = true;
    else isUSB = false;
    ubitx_mode = MODE_SSB;
    setFrequency(frequency);
    updateDisplay();
    startTx(TX_SSB);
    
  }else if( strncmp((char*)CAT_BUFF,"VV:",3) == 0){
    if( vfoActive == VFO_A )vfoB = frequency;
    else {
      vfoB = vfoA;
      frequency = vfoA;
    }
    updateDisplay();
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////

void  Process_CAT_Command( void ){
byte i;

  // wait for command
  if( Serial.available() > 0 ){
    CAT_BUFF[Cat_Buffer_Index] = Serial.read();
    if( CAT_BUFF[Cat_Buffer_Index ] == ';'){
      CAT_BUFF[Cat_Buffer_Index+1 ] = 0;
		  Process_Cat_Command(); 
		  Cat_Buffer_Index = 0;
	  }else{
	    Cat_Buffer_Index = Cat_Buffer_Index +1;
	  }
  }
}

void Process_EEPROM( void ){
byte i;


  printLine2((char*)"EEPROM UPDATE");
  delay(1000);

  // check if button held down go to factory alignment
  i = 0;
  while( btnDown()){
    i++;
    if( i > 75 ){
      printLine2((char*)"ALIGNMENT");
      delay(100);
    }
  }
  
  if( i > 75) doAlignment();
  Init_Cat(38400, SERIAL_8N1);

  while (1){ 
  
    // check to exit via button
    if( btnDown() ) {
      printLine1((char*)"Button");
      i = 0;
      while( btnDown()){
        i++;
        if( i > 75 )printLine2((char*)"Exit");
        delay(10);
      }
      if( i > 75) MenuExit(1);
     
    }else{
      printLine1((char*)"       ");
      Process_CAT_Command();
    }
  }
}


