// File name ubitx_factory_alignment.cpp

 
#include <Arduino.h>
#include <EEPROM.h>
#include "ubitx.h"
#include <LiquidCrystal.h>
extern LiquidCrystal lcd;


extern int  btnDown(void);
extern void printLine1(char *c);
extern void printLine2(char *c);
extern void si5351_set_calibration(int32_t cal);
extern void startTx(byte txMode);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void cwKeydown(void);
extern void cwKeyUp(void);
extern int  enc_read(void);
extern void stopTx(void);
extern void initOscillators(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern void checkPTT(void);
extern void printLine(char linenmbr, char *c);
extern void ritDisable(void);


extern unsigned long usbCarrier;
extern byte menuOn;
extern bool isUSB;
extern char c[30], b[30];
extern volatile bool keyDown;
extern unsigned long cwTimeout;
extern unsigned long cwTimeout;
extern unsigned long frequency;
extern unsigned long Freq_Incr[6];
extern int Freq_Index;
extern int ubitx_mode;
extern char ritOn;
extern char vfoActive;
extern int Freq_Underline[6];


/**
 * This procedure is only for those who have a signal generator/transceiver tuned to exactly 7.150 and a dummy load 
 */

 //this is used by the si5351 routines in the ubitx_5351 file
extern int32_t calibration;
extern uint32_t si5351bx_vcoa;

void Update_Freq(void){
memset(c, 0, sizeof(c));
memset(b, 0, sizeof(b));

  if (isUSB)strcpy(c, "USB ");
  else strcpy(c, "LSB ");
  
  if (ritOn) strcat(c, "r");
  else strcat(c, " ");
  
  if (vfoActive == VFO_A) strcat(c, " A");
  else strcat(c, " B");
  
  ultoa(frequency, b, DEC);
  //one mhz digit if less than 10 M, two digits if more
  if (strlen(b) < 8){
    c[7] = ' ';
    c[8]  = b[0];
    strcat(c, ".");
    strncat(c, &b[1], 3);    
    strcat(c, ".");
    strncat(c, &b[4], 2);
  }
  else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);    
  }


  printLine(1, c);
  lcd.setCursor(Freq_Underline[Freq_Index], 1);

}

void CaldoTuning(){
  int s;
  unsigned long prev_freq;

  s = enc_read();
  if (s){
    prev_freq = frequency;
    
    if (s > 4)
      frequency += Freq_Incr[Freq_Index];
    else if (s >  2)
      frequency += Freq_Incr[Freq_Index];
    else if (s >  0)
      frequency +=  Freq_Incr[Freq_Index];
    else if (s <  0)
      frequency -= Freq_Incr[Freq_Index];
    else if (s < -2)
      frequency -= Freq_Incr[Freq_Index];
    else if (s < -4)
      frequency -= Freq_Incr[Freq_Index];
      
    if ( ubitx_mode == MODE_SSB ){
      if (prev_freq < 10000000l && frequency >= 10000000l) isUSB = true;
      if (prev_freq >= 10000000l && frequency < 10000000l) isUSB = false;
    }
   
    setFrequency(frequency);
    Update_Freq();
    }
}


void menuSetupTuning( int btn ){
int i;
bool looper = true;

  //keep clear of any previous button press
  while (btnDown())
    delay(100);
  delay(100);

  if (!btn){
    printLine2((char *)"Tune station?");
    return;
  }
    printLine2((char *)"Encoder to Exit");
    Update_Freq();

while( looper ){
  //only if the button is pressed
  if (btnDown()){
    delay(100);
    if (btnDown()){
      // OK button is definitely pushed so now detect how long it is pressed.
      i=0;
      while ( btnDown() ){
        delay(10);
        i++;
        if (i > BtnPressTime )printLine2((char *)"Exit Tune");
      }
      if (i <= BtnPressTime ){
        Freq_Index = Freq_Index + 1;
        if( Freq_Index > 4 )Freq_Index = 0;
        Update_Freq();
      }else{
        printLine(1, (char*)"");
        while(btnDown())delay(50);
        delay(100);
        return;
      }
    }
   }
    CaldoTuning();
    delay(100);
}
 
}





int factoryCalibration(int btn){
  int knob = 0;
  int32_t prev_calibration;


  //keep clear of any previous button press
  while (btnDown())
    delay(100);
  delay(100);
   
  if (!btn){
    printLine2((char *)"Set Calibration?");
    return 0;
  }

  prev_calibration = calibration;
  calibration = 0;

  isUSB = true;

  //turn off the second local oscillator and the bfo
  si5351_set_calibration(calibration);
  startTx(TX_CW);
  si5351bx_setfreq(2, 10000000l); 
  
  strcpy(b, "#1 10 MHz cal:");
  ltoa(calibration/8750, c, 10);
  strcat(b, c);
  printLine2(b);     

  while (!btnDown())
  {

    if (digitalRead(PTT) == LOW && !keyDown)
      cwKeydown();
    if (digitalRead(PTT)  == HIGH && keyDown)
      cwKeyUp();
      
    knob = enc_read();

    if (knob > 0)
      calibration += 875;
    else if (knob < 0)
      calibration -= 875;
    else 
      continue; //don't update the frequency or the display
      
    si5351_set_calibration(calibration);
    si5351bx_setfreq(2, 10000000l);
    strcpy(b, "#1 10 MHz cal:");
    ltoa(calibration/8750, c, 10);
    strcat(b, c);
    printLine2(b);     
  }

  cwTimeout = 0;
  keyDown = 0;
  stopTx();

  printLine2((char *)"Calibration set!");
  EEPROM.put(MASTER_CAL, calibration);
  initOscillators();
  setFrequency(frequency);    
  updateDisplay();

  while(btnDown())
    delay(50);
  delay(100);
}

int menuSetupCalibration(int btn){
  int knob = 0;
  int32_t prev_calibration;
   
  if (!btn){
    printLine2((char *)"Set Calibration?");
    return 0;
  }

  printLine1((char *)"Set to Zero-beat,");
  printLine2((char *)"press PTT to save");
  delay(1000);
  
  prev_calibration = calibration;
  calibration = 0;
  si5351_set_calibration(calibration);
  setFrequency(frequency);    
  
  strcpy(b, "cal:");
  ltoa(calibration/8750, c, 10);
  strcat(b, c);
  printLine2(b);     

  while (digitalRead(PTT) == HIGH && !btnDown())
  {
    knob = enc_read();

    if (knob > 0){
      calibration += 8750;
      usbCarrier += 120;
    }
    else if (knob < 0){
      calibration -= 8750;
      usbCarrier -= 120;
    }
    else
      continue; //don't update the frequency or the display

    si5351_set_calibration(calibration);
    si5351bx_setfreq(0, usbCarrier);
    setFrequency(frequency);    

    strcpy(b, "cal:");
    ltoa(calibration/8750, c, 10);
    strcat(b, c);
    printLine2(b);     
  }
  
  //save the setting
  if (digitalRead(PTT) == LOW){
    printLine1((char *)"Calibration set!");
    printLine2((char *)"Set Carrier now");
    EEPROM.put(MASTER_CAL, calibration);
    delay(2000);
  }
  else
    calibration = prev_calibration;

  printLine2((char *)"");
  initOscillators();
  //si5351_set_calibration(calibration);
  setFrequency(frequency);    
  updateDisplay();
}


void printCarrierFreq(unsigned long freq){

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  ultoa(freq, b, DEC);
  
  strncat(c, b, 2);
  strcat(c, ".");
  strncat(c, &b[2], 3);
  strcat(c, ".");
  strncat(c, &b[5], 1);
  printLine2(c);    
}

void menuSetupCarrier(int btn){
  int knob = 0;
  unsigned long prevCarrier;
   
  if (!btn){
      printLine2((char *)"Set the BFO");
    return;
  }

  prevCarrier = usbCarrier;
  printLine1((char *)"Tune to best Signal");  
  printLine2((char *)"PTT to confirm. ");
  delay(1000);

  usbCarrier = 11995000l;
  si5351bx_setfreq(0, usbCarrier);
  printCarrierFreq(usbCarrier);

  //disable all clock 1 and clock 2 
  while (digitalRead(PTT) == HIGH && !btnDown())
  {
    knob = enc_read();

    if (knob > 0)
      usbCarrier -= 50;
    else if (knob < 0)
      usbCarrier += 50;
    else
      continue; //don't update the frequency or the display
      
    si5351bx_setfreq(0, usbCarrier);
    printCarrierFreq(usbCarrier);
    
    delay(100);
  }

  //save the setting
  if (digitalRead(PTT) == LOW){
    printLine2((char *)"Carrier set!    ");
    EEPROM.put(USB_CAL, usbCarrier);
    delay(1000);
  }
  else 
    usbCarrier = prevCarrier;

  si5351bx_setfreq(0, usbCarrier);          
  setFrequency(frequency);    
  updateDisplay();
  printLine2((char *)"");
}

void btnWaitForClick(){
  while(!btnDown())
    delay(50);
  while(btnDown())
    delay(50);
 delay(50);
}

void factory_alignment(){
        
  factoryCalibration(1);

  if (calibration == 0){
    printLine2((char *)"Setup Aborted");
    return;
  }

  //move it away to 7.160 for an LSB signal
  setFrequency(7160000l);
  updateDisplay();
  printLine2((char *)"#2 BFO");
  delay(1000);

  usbCarrier = 11994999l;
  menuSetupCarrier(1);

  if (usbCarrier == 11994999l){
    printLine2((char *)"Setup Aborted");
    return;
  }

  
  printLine2((char *)"#3:Test 3.5MHz");
  isUSB = false;
  setFrequency(3500000l);
  updateDisplay();

  while (!btnDown()){
    checkPTT();
    delay(100);
  }

  btnWaitForClick();
  printLine2((char *)"#4:Test 7MHz");

  setFrequency(7150000l);
  updateDisplay();
  while (!btnDown()){
    checkPTT();
    delay(100);
  }

  btnWaitForClick();
  printLine2((char *)"#5:Test 14MHz");

  isUSB = true;
  setFrequency(14000000l);
  updateDisplay();
  while (!btnDown()){
    checkPTT();
    delay(100);
  }

  btnWaitForClick();
  printLine2((char*)"#6:Test 28MHz");

  setFrequency(28000000l);
  updateDisplay();
  while (!btnDown()){
    checkPTT();
    delay(100);
  }

  printLine2((char*)"Alignment done");
  delay(1000);

  isUSB = false;
  setFrequency(7150000l);
  updateDisplay();  
  
}


void MenuExit(int btn){
  
  if (!btn){
      printLine2((char*)"Exit ");
    return;
  }
  printLine2((char*)"Power Cycle");
  printLine1((char*)"Radio");
  while(1)delay(1000);
}

void doAlignment(void){
  int select=0, i;
  bool btnState;

  //wait for the button to be raised up
  while(btnDown())
    delay(50);
  delay(50);  //debounce


  ubitx_mode = MODE_SSB;
  ritDisable();
  menuOn = 2;
  printLine1((char *)" ");
  while (menuOn){
    i = enc_read();
    btnState = btnDown();

    if (i > 1){
      if (select + i < 40)
        select += i;
    }
    if (i < 0 && select - i >= 0)
      select += i;      //caught ya, i is already -ve here, so you add it

    if (select < 10)
      MenuExit(btnState);
    else if (select < 20)
     menuSetupTuning(btnState);
    else if (select < 30)
     menuSetupCalibration(btnState);   //crystal
    else if (select < 40)
      menuSetupCarrier(btnState);       //lsb
  }

  //debounce the button
  while(btnDown())
    delay(50);
  delay(50);
}


