/** Menus
 *  File Name ubitx_menu.cpp
 *  The Radio menus are accessed by tapping on the function button. 
 *  - The main loop() constantly looks for a button press and calls doMenu() when it detects
 *  a function button press. 
 *  - As the encoder is rotated, at every 10th pulse, the next or the previous menu
 *  item is displayed. Each menu item is controlled by it's own function.
 *  - Each menu function may be called to display itself
 *  - Each of these menu routines is called with a button parameter. 
 *  - The btn flag denotes if the menu item was clicked on or not.
 *  - If the menu item is clicked on, then it is selected,
 *  - If the menu item is NOT clicked on, then the menu's prompt is to be displayed
 */
#include <Arduino.h>
#include <EEPROM.h>
#include "ubitx.h"

#include <LiquidCrystal.h>
extern LiquidCrystal lcd;

extern void stopTx(void);
extern void printLine2(char *c);
extern bool btnDown(void);
extern void ritDisable(void);
extern void ritEnable();
extern int  enc_read(void);
extern void setFrequency(unsigned long f);
extern void updateDisplay(void);
extern void printLine1(char *c);
extern void si5351_set_calibration(int32_t cal);
extern void startTx(byte txMode);
extern void si5351bx_setfreq(uint8_t clknum, uint32_t fout);
extern void cwKeydown(void);
extern void cwKeyUp(void);
extern void initOscillators(void);
extern int Test_Button( void );
extern int Test_Button( char * title );



extern unsigned long frequency;
extern char ritOn;
extern char splitOn;
extern bool isUSB;
extern byte menuOn;
extern char vfoActive;
extern unsigned long vfoA;
extern unsigned long vfoB;
extern boolean modeCalibrate;
extern int cwSpeed;
extern char c[30], b[30];
extern volatile bool keyDown;
extern unsigned long cwTimeout;
extern unsigned long usbCarrier;
extern unsigned long sideTone;
extern bool Iambic_Key;
extern char Version_Text[16];
extern char Version_Date[16];
extern int Menu_Select_Index[7][2];         // new menu
extern int Menu_Index;                      // new menu
extern int ubitx_mode;


/////// band data
int ubitx_band;
int ubitx_last_band;
char ubitx_band_text[8][3] = { "80","40","30","20","17","15","12","10" };
unsigned long ubitx_band_freq[8] = { 3540000L,7040000L,10106000L,14060000L,18096000L,21060000L,23906000L,28060000L };
unsigned long ubitx_SSB_freq[8] = { 3985000L,7285000L,10106000L,14285000L,18130000L,21385000L,24950000L,28385000L };
///////

int wpm;

int Menu_Cursor_Line_Postions[8] = {0,0,0,0,1,1,1,1};
int Menu_Cursor_Index_Postions[8] = {0,3,7,12,0,2,6,7};
int Max_Menus_Items = 7;
int Current_Menu = 0;
int Last_Menu = 0;

////////////////////////////////////////////////////////////

void menuBand( void ){
  int bandmenuOn = 2;
  int i,btnState;
  unsigned long offset;

    
  ubitx_band = 0;
  printLine2(ubitx_band_text[ubitx_band]);
  //wait for the button menu select button to be lifted)
  while (btnDown()) delay(50);
  delay(50);    
  ritDisable();
  
  printLine2(ubitx_band_text[ubitx_band]);
  
  while (bandmenuOn){
    
    i = enc_read();
    if( i != 0){
      if (i > 1){
        ubitx_band++;
        if( ubitx_band > 7) ubitx_band = 7;
      }else if (i < -1 ){
        ubitx_band--;
        if( ubitx_band < 0) ubitx_band = 0;
      }
      printLine2(ubitx_band_text[ubitx_band]);
      delay(500);
    }
    if( btnDown() ){
      while ( btnDown() ) delay(50);
      bandmenuOn = 0;
      menuOn = 0;
      ubitx_last_band = ubitx_band;
    }
 }
 
 if( ubitx_band < 2 ){
   if( ubitx_mode == MODE_CW ){
    setFrequency(ubitx_band_freq[ubitx_band]);
    isUSB = true;
   }else{
    setFrequency(ubitx_SSB_freq[ubitx_band]);
    isUSB = false;
   }
 }else{
   isUSB = true;
   if( ubitx_mode == MODE_CW ) setFrequency(ubitx_band_freq[ubitx_band]);
   else setFrequency(ubitx_SSB_freq[ubitx_band]);
 }
 updateDisplay();
 
}


void menuVfoToggle( void ){
  
  if(splitOn){
      if (vfoActive == VFO_B)vfoActive = VFO_A;
      else vfoActive = VFO_B;
  }else{
      if (vfoActive == VFO_B){
        vfoB = frequency;
        EEPROM.put(VFO_B, frequency);
        vfoActive = VFO_A;
        frequency = vfoA;
      }
      else {
        vfoA = frequency;
        EEPROM.put(VFO_A, frequency);
        vfoActive = VFO_B;
        frequency = vfoB;
      }
  }   
      ritDisable();
      if( ubitx_mode == MODE_SSB){
        setFrequency(frequency);
        if (frequency >= 10000000l)
          isUSB = true;
        else
          isUSB = false;
      }
      updateDisplay();
      delay(200);
      //exit the menu
      menuOn = 0;
  
}


void menuRitToggle( void ){
      if (ritOn == 0){
        ritEnable();
      }else{
        ritDisable();
      }
      menuOn = 0;
      delay(100);
      updateDisplay();
 }

void menuSidebandToggle( void ){
    if (isUSB == true) isUSB = false;
    else isUSB = true;
    delay(200);
        setFrequency(frequency);
    updateDisplay();
    menuOn = 0;
}

void menuModeToggle( void ){
    if (ubitx_mode == MODE_SSB){
      ubitx_mode = MODE_CW;
      isUSB = true;
    }else{
      ubitx_mode = MODE_SSB;
    }
    EEPROM.put(MODE, ubitx_mode );
    delay(200);
    setFrequency(frequency);
    updateDisplay();
    menuOn = 0;
}

void menuKeyToggle( void ){
    if (Iambic_Key){
      Iambic_Key = false;
    }else{
      Iambic_Key = true;
    }
    EEPROM.put(KEY_TYPE, Iambic_Key );
    delay(200);
    updateDisplay();
    menuOn = 0;
}


void menuSplitToggle( void ){

  if( splitOn ){
    printLine2((char*)"split Disable");
    delay(1000);
    splitOn = 0;
    vfoActive = VFO_A;
    frequency = vfoA;
    Last_Menu = 6;
  }else{
  printLine2((char*)"split Enable");
  delay(1000);
    splitOn = 1;
    vfoB = frequency;
    Last_Menu = 6;
  }
    updateDisplay(); 
    menuOn = 0; 
}




void menuCWSpeed( void ){
    int knob = 0;


    strcpy(b,"            ");
    itoa(wpm,c, 10);
    strcat(b, c);
    strcat(b, "wm");
    printLine2(b);
    lcd.setCursor(12,0);
    delay(200);

    while(!btnDown() && digitalRead(PTT) == HIGH){

      knob = enc_read();
      if (knob != 0){
        if (wpm > 3 && knob < 0)
          wpm--;
        if (wpm < 50 && knob > 0)
          wpm++;

    strcpy(b,"            ");
    itoa(wpm,c, 10);
    strcat(b, c);
    strcat(b, "wm");
    printLine2(b);
    lcd.setCursor(12,0);
    delay(200);
      }
      //abort if this button is down
      if (btnDown())
        //re-enable the clock1 and clock 2
        break;
    }
    
    //save the setting
    cwSpeed = 600 / wpm;
    EEPROM.put(CW_SPEED, wpm);
    delay(200);
    menuOn = 0;
 updateDisplay();
}



void menuSetupCwTone( void ){
    int knob = 0;
     
    strcpy(c,"       ");
    itoa(sideTone, b, 10);
    strcat(c,b);
    strcat(c,"H");
    printLine2(c);
    lcd.setCursor(7,0);
    delay(200);
    tone(CW_TONE, sideTone);

    //disable all clock 1 and clock 2 
    while (digitalRead(PTT) == LOW || !btnDown())
    {
      knob = enc_read();

      if (knob > 0 && sideTone < 2000)
        sideTone += 10;
      else if (knob < 0 && sideTone > 100 )
        sideTone -= 10;
      else
        continue; //don't update the frequency or the display
        
      tone(CW_TONE, sideTone);
      strcpy(c,"       ");
      itoa(sideTone, b, 10);
      strcat(c,b);
      strcat(c,"H");
      printLine2(c);
      lcd.setCursor(7,0);
      delay(100);
    }
    noTone(CW_TONE);
    //save the setting
    EEPROM.put(CW_SIDETONE, sideTone);
    delay(200);
    updateDisplay(); 
    menuOn = 0; 
 }


void doMenu(void){
  int i;
  int Do_Something;

  Current_Menu = Last_Menu;
  updateDisplay();
  lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu]);
 
  //wait for the button to be raised up
  while(btnDown()) delay(50);
  delay(50);  //debounce
  
  menuOn = 2;

  while (menuOn){
    Last_Menu = Current_Menu;
  
    // check if user moving to different menu selection    
    i = enc_read();
    if( i != 0){
      if( ritOn ) Current_Menu = 4;
      else if( splitOn ) Current_Menu = 6;
      else{
        if (i > 1) {
          if( ubitx_mode == MODE_SSB){
            if (Current_Menu == 0 )Current_Menu = 4;
            else Current_Menu = Current_Menu + 1;
            if (Current_Menu > Max_Menus_Items ) Current_Menu = 0;
          }else{
            if( (Iambic_Key == false) && (Current_Menu == 2))Current_Menu = 4;
            else Current_Menu = Current_Menu + 1;
            if (Current_Menu > Max_Menus_Items ) Current_Menu = 0;
          }
          lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu] );
        }else if (i < -1 ){
          if( ubitx_mode == MODE_SSB){
            if (Current_Menu == 4 )Current_Menu = 0;
            else Current_Menu = Current_Menu - 1;
            if (Current_Menu < 0 ) Current_Menu = Max_Menus_Items;
          }else{
            if( (Iambic_Key == false) && (Current_Menu == 4))Current_Menu = 2;
            else Current_Menu = Current_Menu - 1;
            if (Current_Menu < 0 ) Current_Menu = Max_Menus_Items;
          }
        }
      }
      lcd.setCursor(Menu_Cursor_Index_Postions[Current_Menu],Menu_Cursor_Line_Postions[Current_Menu] );
      delay(500);

    }else{
    
  Do_Something = Test_Button((char*)"Exit Menu");
  if( Do_Something > 0 ){
    menuOn = 0;
  }else if( Do_Something == -1 ){

    switch  ( Current_Menu ) {
      
      case  (0):
        menuModeToggle();
        break;
      case  (1):
        menuKeyToggle();
        break;
      case  (2):
        menuSetupCwTone();
        break;
      case  (3):
        menuCWSpeed();
        break;
      case  (4):
        menuRitToggle();
        break;
      case  (5):
        menuSidebandToggle();
        break;
      case  (6):
        menuVfoToggle();
        break;
      case  (7):
        menuBand();
        break;
    }
  }
  //debounce the button
  while(btnDown()) delay(50);
  delay(50);
  
    }}
      updateDisplay();

    }

