/**
 * ubitx_ui.cpp
 * The user interface of the uBITX consists of the encoder, the push-button on top of it
 * and the 16x2 LCD display.
 * The upper line of the display is constantly used to display frequency and status
 * of the radio. Occasionally, it is used to provide a two-line information that is 
 * quickly cleared up.
 */

//      
#include <Arduino.h>
#include "ubitx.h"
#include <LiquidCrystal.h>
extern LiquidCrystal lcd;


extern void printLine1(char *c);


extern char printBuff[2][17];
extern char c[30], b[30];
extern unsigned long frequency;
extern unsigned long sideTone;    // new menu
extern char inTx;
extern unsigned long cwTimeout;
extern char ritOn;
extern char splitOn;
extern bool isUSB;
extern char vfoActive;
extern int Freq_Underline[6];
extern int Freq_Index;
extern volatile int ubitx_mode;
extern int cwSpeed;
extern int wpm;
extern bool Iambic_Key;
extern unsigned long vfoA;
extern unsigned long vfoB;
extern long ritOffset;


/////////////////////////////////////////////////////////////////////////////


//returns true if the button is pressed
bool btnDown(void){
  if (digitalRead(FBUTTON) == HIGH){
    return false;
  }else{
   return true;
  }
}


// The generic routine to display one line on the LCD 
void printLine(char linenmbr, char *c) {
    uint8_t oldSREG = SREG;
   sei();
  
  if (strcmp(c, printBuff[linenmbr])) {     // only refresh the display when there was a change
    lcd.setCursor(0, linenmbr);             // place the cursor at the beginning of the selected line
    lcd.print(c);
    strcpy(printBuff[linenmbr], c);

    for (byte i = strlen(c); i < 16; i++) { // add white spaces until the end of the 16 characters line is reached
      lcd.print(' ');
    }
  }
  SREG=oldSREG;
}

//  short cut to print to the first line
void printLine3(char *c){
  printLine(2,c);
}

//  short cut to print to the first line
void printLine4(char *c){
  printLine(3,c);
}

//  short cut to print to the first line
void printLine1(char *c){
  printLine(1,c);
}
//  short cut to print to the first line
void printLine2(char *c){
  printLine(0,c);
}

// this builds up the top line of the display with frequency and mode
void updateDisplay(void) {
  char local_buf[12];
  unsigned long temp;
  // tks Jack Purdum W8TEE
  // replaced fsprint commands by str commands for code size reduction


  //now, the second line
  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  if( ubitx_mode == MODE_SSB )strcpy(c, "SSB:");
  else{
    strcpy(c, "CW:");
    if( Iambic_Key ){
      strcat(c, "Pdl ");
      itoa(sideTone,b,10);
      strcat(c,b);
      strcat(c,"H ");
      itoa(wpm,b,10);
      strcat(c,b);
      strcat(c,"wm");
    }else{
      strcat(c, "Key ");
      itoa(sideTone,b,10);
      strcat(c,b);
      strcat(c,"H ");
    }
  }
  if(splitOn){
    if( vfoActive == VFO_B )strcpy(&c[3],"   T");
    else strcpy(&c[3],"   t");
    ultoa(vfoB, b, DEC);
    if (strlen(b) < 7){
      strcat(&c[7], "00.");
      strncat(c, &b[0], 3);
      strcat(c, ".");
      strncat(c, &b[3], 2);    
    }else if (strlen(b) < 8){
      strcat(&c[7], "0");
      strncat(c, &b[0], 1);
      strcat(c, ".");
     strncat(c, &b[1], 3);    
      strcat(c, ".");
      strncat(c, &b[4], 2);
    }else {
      strncat(c, b, 2);
      strcat(c, ".");
      strncat(c, &b[2], 3);
      strcat(c, ".");
      strncat(c, &b[5], 2);    
    }
  }else if (ritOn){
    if( ubitx_mode == MODE_SSB ) strcpy(c, "SSB      ");
    else strcpy(c,"CW       ");
    if( ritOffset < 0 ){
      strcat(c, "R-");
      temp = 0 - ritOffset;
    }else{
      strcat(c, "R+");
      temp = ritOffset;
    }
    ultoa(temp/100, b, DEC);
    if( strlen(b) == 1){
      strcat(c,"00.");
      strncat(c,&b[0],1);
    }
    if( strlen(b) == 2){
      strcat(c,"0");
      strncat(c,&b[0],1);
      strcat(c,".");
      strncat(c,&b[1],1);
    }
    if( strlen(b) == 3){
      strncat(c,&b[0],2);
      strcat(c,".");
      strncat(c,&b[2],1);
    }
  }
  printLine2(c);

  memset(c, 0, sizeof(c));
  memset(b, 0, sizeof(b));

  strcpy(c, "R ");

  if (isUSB)strcat(c, "USB ");
  else strcat(c, "LSB ");
  
  if ( splitOn){
    if( vfoActive == VFO_A )strcat(c,"R");
    else strcat(c,"r");
  }
  else if(vfoActive == VFO_A )strcat(c, "A");
  else strcat(c, "B");
  
  ultoa(frequency, b, DEC);
  if (strlen(b) < 7){
    strcat(&c[7], "00.");
    strncat(c, &b[0], 3);
    strcat(c, ".");
    strncat(c, &b[3], 2);    
  }else if (strlen(b) < 8){
    strcat(&c[7], "0");
    strncat(c, &b[0], 1);
    strcat(c, ".");
    strncat(c, &b[1], 3);    
    strcat(c, ".");
    strncat(c, &b[4], 2);
  }else {
    strncat(c, b, 2);
    strcat(c, ".");
    strncat(c, &b[2], 3);
    strcat(c, ".");
    strncat(c, &b[5], 2);    
  }


  printLine(1, c);
  if (!ritOn){
    uint8_t oldSREG = SREG;
   sei();
    if((splitOn)&&(vfoActive == VFO_B))lcd.setCursor(Freq_Underline[Freq_Index], 0);
    else lcd.setCursor(Freq_Underline[Freq_Index], 1);
  SREG=oldSREG;
  }
}

int enc_prev_state = 3;

/**
 * The A0 And A1 are purely analog lines on the Arduino Nano
 * These need to be pulled up externally using two 10K resistors
 * 
 * There are excellent pages on the Internet about how these encoders work
 * and how they should be used. We have elected to use the simplest way
 * to use these encoders without the complexity of interrupts etc to 
 * keep it understandable.
 * 
 * The enc_state returns a two-bit number such that each bit reflects the current
 * value of each of the two phases of the encoder
 * 
 * The enc_read returns the number of net pulses counted over 50 msecs. 
 * If the puluses are -ve, they were anti-clockwise, if they are +ve, the
 * were in the clockwise directions. Higher the pulses, greater the speed
 * at which the enccoder was spun
 */

byte enc_state (void) {
    return (analogRead(ENC_A) > 500 ? 1 : 0) + (analogRead(ENC_B) > 500 ? 2: 0);
}

int enc_read(void) {
  int result = 0; 
  byte newState;
  int enc_speed = 0;
  
  long stop_by = millis() + 50;
  
  while (millis() < stop_by) { // check if the previous state was stable
    newState = enc_state(); // Get current state  
    
    if (newState != enc_prev_state)
      delay (1);
    
    if (enc_state() != newState || newState == enc_prev_state)
      continue; 
    //these transitions point to the encoder being rotated anti-clockwise
    if ((enc_prev_state == 0 && newState == 2) || 
      (enc_prev_state == 2 && newState == 3) || 
      (enc_prev_state == 3 && newState == 1) || 
      (enc_prev_state == 1 && newState == 0)){
        //result--;
        result++;
      }
    //these transitions point o the enccoder being rotated clockwise
    if ((enc_prev_state == 0 && newState == 1) || 
      (enc_prev_state == 1 && newState == 3) || 
      (enc_prev_state == 3 && newState == 2) || 
      (enc_prev_state == 2 && newState == 0)){
        //result++;
        result--;
      }
    enc_prev_state = newState; // Record state for next pulse interpretation
    enc_speed++;
    delay(1);
  }
  return(result);
}


